@@ -5,11 +5,13 @@ import (
 	"sync"
 	"time"
 
+	v1 "k8s.io/api/core/v1"
 	"k8s.io/klog"
 
 	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
 	"k8s.io/apimachinery/pkg/util/sets"
 	utilwait "k8s.io/apimachinery/pkg/util/wait"
+	kcoreinformers "k8s.io/client-go/informers/core/v1"
 	"k8s.io/client-go/util/retry"
 
 	networkclient "github.com/openshift/client-go/network/clientset/versioned"
@@ -23,6 +25,7 @@ type egressIPManager struct {
 	tracker            *common.EgressIPTracker
 	networkClient      networkclient.Interface
 	hostSubnetInformer networkinformers.HostSubnetInformer
+	nodeInformer       kcoreinformers.NodeInformer
 
 	updatePending bool
 	updatedAgain  bool
@@ -33,6 +36,7 @@ type egressIPManager struct {
 
 type egressNode struct {
 	ip      string
+	name    string
 	offline bool
 	retries int
 }
@@ -43,9 +47,10 @@ func newEgressIPManager() *egressIPManager {
 	return eim
 }
 
-func (eim *egressIPManager) Start(networkClient networkclient.Interface, hostSubnetInformer networkinformers.HostSubnetInformer, netNamespaceInformer networkinformers.NetNamespaceInformer) {
+func (eim *egressIPManager) Start(networkClient networkclient.Interface, hostSubnetInformer networkinformers.HostSubnetInformer, netNamespaceInformer networkinformers.NetNamespaceInformer, nodeInformer kcoreinformers.NodeInformer) {
 	eim.networkClient = networkClient
 	eim.hostSubnetInformer = hostSubnetInformer
+	eim.nodeInformer = nodeInformer
 	eim.tracker.Start(hostSubnetInformer, netNamespaceInformer)
 }
 
@@ -98,7 +103,7 @@ func (eim *egressIPManager) maybeDoUpdateEgressCIDRs() (bool, error) {
 			if node := eim.monitorNodes[hs.HostIP]; node != nil {
 				monitorNodes[hs.HostIP] = node
 			} else {
-				monitorNodes[hs.HostIP] = &egressNode{ip: hs.HostIP}
+				monitorNodes[hs.HostIP] = &egressNode{ip: hs.HostIP, name: nodeName}
 			}
 
 			oldIPs := sets.NewString(hs.EgressIPs...)
@@ -146,7 +151,10 @@ func (eim *egressIPManager) poll(stop chan struct{}) {
 		}
 
 		start := time.Now()
-		retry := eim.check(retry)
+		retry, err := eim.check(retry)
+		if err != nil {
+			klog.Warningf("Node may have been deleted or not exist anymore")
+		}
 		if !retry {
 			// If less than pollInterval has passed since start, then sleep until it has
 			time.Sleep(start.Add(pollInterval).Sub(time.Now()))
@@ -154,7 +162,19 @@ func (eim *egressIPManager) poll(stop chan struct{}) {
 	}
 }
 
-func (eim *egressIPManager) check(retrying bool) bool {
+func nodeIsReady(node *v1.Node) bool {
+	nodeReady := true
+	for _, cond := range node.Status.Conditions {
+		if cond.Type == v1.NodeReady {
+			if cond.Status == v1.ConditionFalse || cond.Status == v1.ConditionUnknown {
+				nodeReady = false
+			}
+		}
+	}
+	return nodeReady
+}
+
+func (eim *egressIPManager) check(retrying bool) (bool, error) {
 	var timeout time.Duration
 	if retrying {
 		timeout = repollInterval
@@ -168,6 +188,19 @@ func (eim *egressIPManager) check(retrying bool) bool {
 			continue
 		}
 
+		nn, err := eim.nodeInformer.Lister().Get(node.name)
+		if err != nil {
+			return false, err
+		}
+
+		if !nodeIsReady(nn) {
+			klog.Warningf("Node %s is not Ready", node.name)
+			node.offline = true
+			eim.tracker.SetNodeOffline(node.ip, true)
+			// Return when there's a not Ready node
+			return false, nil
+		}
+
 		online := eim.tracker.Ping(node.ip, timeout)
 		if node.offline && online {
 			klog.Infof("Node %s is back online", node.ip)
@@ -187,7 +220,7 @@ func (eim *egressIPManager) check(retrying bool) bool {
 		}
 	}
 
-	return needRetry
+	return needRetry, nil
 }
 
 func (eim *egressIPManager) Synced() {
